
#define REQ_BACK 101
#include "palette.h"
#include "gui.h"
#include "pcx.h"
#include "dirs.h"
#include "mymem.h"
#include "keys.h"
#include "ed2.h"
#include <stdio.h>
#include <string.h>
#include <dirent.h>
#include <X11/Xlib.h>
#include "graph.h"


void
DrawFilledBox(XImage *vscreen,int x, int y, int x1, int y1, int col)
{
	int i, j;
	for (i=x;i<=x1;i++){
		for (j=y;j<=y1;j++){
			PutVPix(vscreen,i,j,col);
		}
	}
}

void 
AddButton(Display *display,struct Button *ThisButton,int x,int y,
               int w,int h, char *imagename,int udPos,int PaletteOffset,
               Window ParentWin,GC gc,int Flags)
{
    Window tmpwin;
    char tmp[40];
    strcpy(tmp,imagename);
    tmpwin=XCreateSimpleWindow(display,ParentWin,x,y,w,h,0,
                               0,0);
    XSelectInput(display, tmpwin, ButtonPressMask|ButtonReleaseMask);
    XMapWindow(display,tmpwin);
    ThisButton->ButtonWin=tmpwin;  
    ThisButton->Flags=Flags; 
    ThisButton->Pending=0; 
    ThisButton->Pressed=0;  
    ThisButton->UpImg=WLoad_PCX(tmp,0,display,PaletteOffset);
    ThisButton->UpPix=XCreatePixmap(display,ParentWin,w,h,8);
    XPutImage(display,ThisButton->UpPix,gc,ThisButton->UpImg,0,0,0,0,w,h);
    tmp[udPos]='d';
    ThisButton->radio[0]=NULL;
    ThisButton->radio[1]=NULL;
    ThisButton->radio[2]=NULL;
    ThisButton->radio[3]=NULL;
    ThisButton->DownImg=WLoad_PCX(tmp,0,display,PaletteOffset);
	if(ThisButton->DownImg) {
    	ThisButton->DownPix=XCreatePixmap(display,ParentWin,w,h,8);
    	XPutImage(display,ThisButton->DownPix,gc,ThisButton->DownImg,0,0,0,0,w,h);
		WFreePic(ThisButton->DownImg);
	}
 	WFreePic(ThisButton->UpImg);
    XSetWindowBackgroundPixmap(display,ThisButton->ButtonWin,ThisButton->UpPix);
    XClearWindow(display,ThisButton->ButtonWin);
}
   
void 
Radio3(struct Button Controls[],int a,int b,int c)
{
    Controls[a].radio[0]=&Controls[b];
    Controls[a].radio[1]=&Controls[c];
    Controls[b].radio[0]=&Controls[a];
    Controls[b].radio[1]=&Controls[c];
    Controls[c].radio[0]=&Controls[a];
    Controls[c].radio[1]=&Controls[b];
}
 
void 
Radio4(struct Button Controls[],int a,int b,int c,int d)
{
    Controls[a].radio[0]=&Controls[b];
    Controls[a].radio[1]=&Controls[c];
    Controls[a].radio[2]=&Controls[d];
    Controls[b].radio[0]=&Controls[a];
    Controls[b].radio[1]=&Controls[c];
    Controls[b].radio[2]=&Controls[d];
    Controls[c].radio[0]=&Controls[a];
    Controls[c].radio[1]=&Controls[b];
    Controls[c].radio[2]=&Controls[d];
    Controls[d].radio[0]=&Controls[a];
    Controls[d].radio[1]=&Controls[b];
    Controls[d].radio[2]=&Controls[c];
}
 
char 
KeyCode2Ascii(Display *display,uchar keycode,uchar state)
{
    static char lookup[2][127]=  {
							 "*********1234567890-=*<*"
                             "qwertyuiop[]#*asdfghjkl;'***"
                             "\\zxcvbnm,./********"
                             "****************"
                             "****************"
                             "****************"
                             "******",
    						 "*********!\"*$%^&*()_+*<*"
                             "QWERTYUIOP{}~*ASDFGHJKL:@***"
                             "*ZXCVBNM***********"
                             "****************"
                             "****************"
                             "****************"
                             "******"
	};
	state&=1;
    if(lookup[state][keycode]!='*') {
        return(lookup[state][keycode]);
    }
    return(-1);
}

void
AffectString(char *str,char letter,int *Curs)
{
    char tmp[100];
    if(letter=='<') {
		if(*Curs>0) {
			strcpy(&str[(*Curs)-1],&str[*Curs]);
			(*Curs)--;
		}
    } else {
        strcpy(tmp,&str[*Curs]);
        str[*Curs]=letter;
        strcpy(&str[++(*Curs)],tmp);
    }
}

void 
KillWindow(Display *display,Window win)
{
    XUnmapWindow(display,win);
    XDestroyWindow(display,win);
}
 
void 
DrawTextBox(Display *display,Window win,GC gc,int BaseColour,
                 int x,int y,int w,int h)
{
    XSetForeground(display,gc,BaseColour+4);
    XDrawLine(display,win,gc,x+1,y+1,x+w-3,y+1);
    XDrawLine(display,win,gc,x+1,y+1,x+1,y+h-3);
    XDrawLine(display,win,gc,x+1,y+h-1,x+w-1,y+h-1);
    XDrawLine(display,win,gc,x+w-1,y+1,x+w-1,y+h-1);
    XSetForeground(display,gc,BaseColour-2);
    XDrawLine(display,win,gc,x,y,x+w-2,y);
    XDrawLine(display,win,gc,x,y,x,y+h-2);
    XDrawLine(display,win,gc,x+2,y+h-2,x+w-2,y+h-2);
    XDrawLine(display,win,gc,x+w-2,y+2,x+w-2,y+h-2);
    XSync(display,False);
}

void
Draw3DBox(Display *d,Window win,GC gc,int BaseColour,int x,int y,int w,int h,
		  int inv)
{
	int inv1=0,inv2=0;
	BaseColour+=4;
	if(inv) {
		inv1=6;
	} else {
		inv2=6;
	}
	XSetForeground(d,gc,BaseColour-inv1);
	XDrawLine(d,win,gc,x,y,x+w-1,y);
	XDrawLine(d,win,gc,x,y,x,y+h-1);
	XSetForeground(d,gc,BaseColour-inv2);
	XDrawLine(d,win,gc,x+w-1,y+h-1,x+w-1,y);
	XDrawLine(d,win,gc,x+w-1,y+h-1,x,y+h-1);
}

void
Draw3DBox1(Display *d,Window win,GC gc,int x,int y,int w,int h,int Colour1,int Colour2,
		  int inv)
{
	int tmp=0;
	if(inv) {
		tmp=Colour1;
		Colour1=Colour2;
		Colour2=tmp;
	}
	XSetForeground(d,gc,Colour1);
	XDrawLine(d,win,gc,x,y,x+w-1,y);
	XDrawLine(d,win,gc,x,y,x,y+h-1);
	XSetForeground(d,gc,Colour2);
	XDrawLine(d,win,gc,x+w-1,y+h-1,x+w-1,y);
	XDrawLine(d,win,gc,x+w-1,y+h-1,x,y+h-1);
}

void
Draw3DBox2(XImage *vscreen,int x,int y,int w,int h,int Colour1,int Colour2,
			int inv)
{
	if(inv) {
		int tmp;
		tmp=Colour1;
		Colour1=Colour2;
		Colour2=tmp;
	}
	HLine(vscreen->data,x,y,w,Colour1);
	VLine(vscreen->data,x,y,h,Colour1);
	HLine(vscreen->data,x,y+h-1,w,Colour2);
	VLine(vscreen->data,x+w-1,y,h,Colour2);
}

void DisplayMotionEditorStatus(Display *display,Window win, GC gc, 
                        struct ControlInfo *control, int BaseColour,
                        Dungeon *dung)
{
    char tmp[30]="";
    XSetForeground(display,gc,BaseColour);
    XSetBackground(display,gc,BaseColour+MID_GRAY);
    sprintf(tmp,"Motion Editor");
    XDrawImageString(display,win,gc,15,402,tmp,strlen(tmp));
    sprintf(tmp,"Move: %2d   ",control->MoveNum);
    XDrawImageString(display,win,gc,15,422,tmp,strlen(tmp));
    
    sprintf(tmp,"Object: %2d  ",control->ActiveObject->ID);
    XDrawImageString(display,win,gc,15,442,tmp,strlen(tmp));
    
     sprintf(tmp,"              ");
    XDrawImageString(display,win,gc,15,462,tmp,strlen(tmp));
     sprintf(tmp,"              ");
    XDrawImageString(display,win,gc,15,482,tmp,strlen(tmp));
     sprintf(tmp,"              ");
    XDrawImageString(display,win,gc,15,502,tmp,strlen(tmp));
     sprintf(tmp,"              ");
    XDrawImageString(display,win,gc,15,522,tmp,strlen(tmp));
     sprintf(tmp,"              ");
    XDrawImageString(display,win,gc,15,542,tmp,strlen(tmp));
     sprintf(tmp,"              ");
    XDrawImageString(display,win,gc,15,562,tmp,strlen(tmp));
     sprintf(tmp,"              ");
    XDrawImageString(display,win,gc,15,582,tmp,strlen(tmp));
     sprintf(tmp,"              ");
    XDrawImageString(display,win,gc,15,602,tmp,strlen(tmp));
     sprintf(tmp,"              ");
    XDrawImageString(display,win,gc,15,622,tmp,strlen(tmp));

    XDrawImageString(display,win,gc,15,642,tmp,strlen(tmp));
}

void 
DisplayEditorStatus(Display *display,Window win,GC gc,
            struct ControlInfo *control, int BaseColour,
            Dungeon *dung )
{
    char tmp[30];

	XSetForeground(display,gc,BaseColour);
    XSetBackground(display,gc,BaseColour+MID_GRAY);
    
    sprintf(tmp,"Character Ed.");
    XDrawImageString(display,win,gc,15,402,tmp,strlen(tmp));
    
    XDrawImageString(display,win,gc,15,422,"Mode:",6);
    switch (control->Mode) {
	    case 0: XDrawImageString(display,win,gc,60,422,"Point  ",8);break;
	    case 1: XDrawImageString(display,win,gc,60,422,"Object ",8);break;
	    case 2: XDrawImageString(display,win,gc,60,422,"Polygon",8);break; 
	    default : break; 
	} 
    XDrawImageString(display,win,gc,15,442,"View:",6);
    switch (control->ShowMode) {
	    case 0: XDrawImageString(display,win,gc,60,442,"Point",6);break;
	    case 1: XDrawImageString(display,win,gc,60,442,"Wire1",6);break;
	    case 2: XDrawImageString(display,win,gc,60,442,"Wire2",6);break; 
        case 3: XDrawImageString(display,win,gc,60,442,"Solid",6);break; 
	    default : break; 
	}
    XDrawImageString(display,win,gc,15,462,"Scale: ",10);
    
    sprintf(tmp,"%2.1f",control->ScaleFactor); 
    XDrawImageString(display,win,gc,80,462,tmp,strlen(tmp));
    
    sprintf(tmp,"Num PiP: %3d",control->newpoly[0]);
    XDrawImageString(display,win,gc,15,482,tmp,strlen(tmp));
    
    sprintf(tmp,"Pts: %4d, %4d,",control->newpoly[1],
        control->newpoly[2]);    
    XDrawImageString(display,win,gc,15,502,tmp,strlen(tmp)); 
    sprintf(tmp,"   : %4d, %4d",control->newpoly[3],
        control->newpoly[4]);
    XDrawImageString(display,win,gc,15,522,tmp,strlen(tmp)); 
    
    sprintf(tmp,"On Poly: %3d",control->ActivePoly);
	XDrawImageString(display,win,gc,15,542,tmp,strlen(tmp)); 
    
     sprintf(tmp,"On Pt: %3d",control->ActivePoint);
    XDrawImageString(display,win,gc,15,562,tmp,strlen(tmp)); 
    sprintf(tmp,"X:%4d", dung->twoDX[control->ActivePoint]);
    XDrawImageString(display,win,gc,15,582,tmp,strlen(tmp));  
    sprintf(tmp,"Y:%4d", dung->twoDY[control->ActivePoint]);
    XDrawImageString(display,win,gc,15,602,tmp,strlen(tmp));  
    sprintf(tmp,"Z:%4d", dung->twoDZ[control->ActivePoint]);
    XDrawImageString(display,win,gc,15,622,tmp,strlen(tmp));   
     
    if (dung->NumPolygons)
    { 
        switch(dung->PolygonArray[control->ActivePoly]->Style) {
            case 0:  XDrawImageString(display,win,gc,15,642,"Style: Col ",11);   
                    break;
            case 1:  XDrawImageString(display,win,gc,15,642,"Style: Text",11);   
                    break;
            case 2:  XDrawImageString(display,win,gc,15,642,"Style: Lit ",11);   
                    break;
            default : break;
        } 
    }
    
// sprintf(tmp,"MovLen: %3d", dung->MoveLength);
//    XDrawImageString(display,win,gc,15,642,tmp,11);      
}

void
DisplayStatus(Display *display,Window win,GC gc,
            struct ControlInfo *control, int BaseColour,
            Dungeon *dung )
{
	if (control->MotionEditor) DisplayMotionEditorStatus(display,win, gc,
            control, BaseColour,dung);
    else {
        DisplayEditorStatus(display,win, gc,
            control, BaseColour,dung);
    }
     XSync(display,False);
}

void
SunkPinkBox(Display *display,Window win,GC gc,int BasePix,int x,int y,int w,int h) 
{
    Draw3DBox1(display,win,gc,x,y,w,h,BasePix+DARK_GRAY,BasePix+WHITE,0);
    XSetForeground(display,gc,BasePix+REQ_BACK);
    XFillRectangle(display,win,gc,x+1,y+1,w-2,h-2);
}
     
void 
DrawRequesterStuff(Display *display,Window win,GC gc,int BaseColour)
{
	//	Files List
	SunkPinkBox(display,win,gc,BaseColour,25,45,250,228);
	//Draw3DBox1(display,win,gc,25,45,250,228,BaseColour+DARK_GRAY,BaseColour+WHITE,0);
	//XSetForeground(display,gc,BaseColour+REQ_BACK);
	//XFillRectangle(display,win,gc,26,46,248,226);

	//	TextBox 1
	Draw3DBox1(display,win,gc,25,295,250,28,BaseColour+DARK_GRAY,BaseColour+WHITE,0);
	XSetForeground(display,gc,BaseColour+REQ_BACK);
	XFillRectangle(display,win,gc,26,296,248,26);

	//	TextBox 2
	Draw3DBox1(display,win,gc,25,345,250,28,BaseColour+DARK_GRAY,BaseColour+WHITE,0);
	XSetForeground(display,gc,BaseColour+REQ_BACK);
	XFillRectangle(display,win,gc,26,346,248,26);

	
	Draw3DBox1(display,win,gc,0,0,300,470,BaseColour+DARK_GRAY,BaseColour+WHITE,1);
	Draw3DBox1(display,win,gc,1,1,298,395,BaseColour+DARK_GRAY,BaseColour+WHITE,1);
	Draw3DBox1(display,win,gc,1,396,298,73,BaseColour+DARK_GRAY,BaseColour+WHITE,1);

    XSync(display,False);
}

void
DrawMessageStuff(Display *display,Window win,GC gc,int BasePix)
{
	SunkPinkBox(display,win,gc,BasePix,25,45,250,108);

	Draw3DBox1(display,win,gc,0,0,300,300,BasePix+DARK_GRAY,BasePix+WHITE,1);
	Draw3DBox1(display,win,gc,1,1,298,225,BasePix+DARK_GRAY,BasePix+WHITE,1);
	Draw3DBox1(display,win,gc,1,226,298,73,BasePix+DARK_GRAY,BasePix+WHITE,1);
	XSetBackground(display,gc,BasePix+REQ_BACK);
	XSetForeground(display,gc,BasePix);
	XDrawImageString(display,win,gc,30,55,"Argh...",7);
}
 
void 
RefreshText(Display *display,Window win,GC gc,int Colour,
                 int x,int y,char *text,int CursX,int ActiveFlag)
{
    char tmp[100];
    int j,k;
    if(CursX==-1) {
	/* Draw a big box  */
        XSetForeground(display,gc,Colour);
		XFillRectangle(display,win,gc,29,y+1,240,19);
        XSetForeground(display,gc,Colour+REQ_BACK);
		XFillRectangle(display,win,gc,30,y+2,238,17);
        //XSetForeground(display,gc,Colour+MID_GRAY);
        XSetBackground(display,gc,Colour);
    } else {
        XSetForeground(display,gc,Colour+REQ_BACK);
		XFillRectangle(display,win,gc,29,y+1,240,19);
        XSetBackground(display,gc,Colour+REQ_BACK);
        XSetForeground(display,gc,Colour);
    }
    strcpy(tmp,"                            ");
    for(j=0;j<strlen(text);j++) {
        tmp[j]=text[j];
    }
	x-=3;
	k=0;
	if(strlen(text)>29) {
		k=strlen(text)-29;
	}
    XDrawImageString(display,win,gc,x+5,y+15,&tmp[k],29);
    //XDrawImageString(display,win,gc,x+5+(8*strlen(text)),y+15," ",1);
    if(ActiveFlag && strlen(text)>=CursX) {
        // Draw Cursor
        XSetBackground(display,gc,Colour);
        XSetForeground(display,gc,Colour+MID_GRAY);
        XDrawImageString(display,win,gc,x+5+(CursX*8),y+15,&text[CursX],1);
        if(CursX==strlen(text)) {
            XSetBackground(display,gc,Colour+REQ_BACK);
            XSetForeground(display,gc,Colour);
            XDrawImageString(display,win,gc,x+14+(8*strlen(text)),y+15," ",1);
        }
    }
}
 
int 
InBox(int x,int y,int l,int t,int r,int b)
{
    if(x>=l && x<=r && y>=t && y<=b) {
		return(1);
	}
    return(0);
}
 
struct DirList *
MakeDirList(void)
{
    struct DirList *tmp;
    tmp=MyMalloc(sizeof(struct DirList),"DirList");
    if(tmp) {
        tmp->Length=0;
        tmp->First=NULL;
    } else {
		fprintf(stdout,"Ohhh shit\n");
	}
    return(tmp);
}

int 
IsDir(char *pwd,char *name)
{
	DIR *tmp;
	char fullname[100];
	strcpy(fullname,pwd);
	if(fullname[strlen(fullname)-1]!='/') {
		strcat(fullname,"/");
	}
	strcat(fullname,name);
	tmp=opendir(fullname);
	if(tmp) {
		closedir(tmp);
		return(1);
	}
	return(0);
}


struct DListNode *
MakeNode(char *filename,struct DListNode *Next,struct DListNode *Prev,int Adir)
{
	struct DListNode *tmp;
	tmp=MyMalloc(sizeof(struct DListNode),"Dlistnode");
	if(tmp) {
		strcpy(tmp->name,filename);
		if(Adir) {
			strcat(tmp->name,"/");
		}
		tmp->Next=Next;
		tmp->Prev=Prev;
	}
	return(tmp);
}

void 
AddEntry(struct DirList *List,struct dirent *entry,char *pwd,char *filter)
{
    struct DListNode *prev=NULL;
    struct DListNode *tmp;
	int OK=0;
	int Adir=0;
	int Added=0;

	if(strlen(entry->d_name)>4) {
		if(!strcmp(filter,&(entry->d_name[strlen(entry->d_name)-4]))) {
			OK=1;
		}
	}
	if(!OK) {
		if(!IsDir(pwd,entry->d_name)) {
			return;
		} else {
			Adir=1;
			OK=1;
		}
	}
	if(!List) {
		//fprintf(stdout,"It was here\n");
	}	
	tmp=List->First;
	if(!tmp) {
		// No List ...
		List->First=MakeNode(entry->d_name,NULL,NULL,Adir);
	} else {
		// there are some entries.
		while(tmp) {
			if(strcmp(entry->d_name,tmp->name)<0 && !Added) {
				Added=1;
				if(!prev) {
					// This is first...
					List->First=MakeNode(entry->d_name,List->First,NULL,Adir);
				} else {
					// Just Stick it in...
					tmp->Prev->Next=MakeNode(entry->d_name,tmp->Prev->Next,tmp->Prev,Adir);
					tmp->Prev->Next->Next->Prev=tmp->Prev->Next;
				}
			}
			prev=tmp;
			tmp=tmp->Next;
		}
		if(!Added) {
			// Add at end.
			prev->Next=MakeNode(entry->d_name,NULL,prev,Adir);
		}
	}
	List->Length++;
}
 
void 
FreeEntries(struct DListNode *node)
{
    if(node->Next) {
        FreeEntries(node->Next);
    }
    MyFree(node,"node");
}
 
void 
FreeDirList(struct DirList *List)
{
    if(List) {
		if(List->First) {
        	FreeEntries(List->First);
		}
        MyFree(List,"List");
    }
}
 
struct DListNode *
GetPtr(struct DirList *DL,int Pos)
{
    struct DListNode *tmp;
    //if(Pos+10>DL->Length) {
    if(Pos>DL->Length) {
        fprintf(stdout,"Returning NULL\n");
        return(NULL);
    } else {
        tmp=DL->First;
        while(Pos--) {
            tmp=tmp->Next;
        }
    }
    return(tmp);
}
   
int 
RefreshList(Display *display,Window win,GC gc,struct DirList *List,
                int ListPos,int Colour,int ListSel)
{
    struct DListNode *top=NULL;
    char tmp[40];
    int i=0,j;
    //fprintf(stdout,"UnRefreshed\n");
    if(List) {
        top=GetPtr(List,ListPos);
        while(i++<11 && top) {
            strcpy(tmp,"                        ");
            for(j=0;j<strlen(top->name);j++) {
                tmp[j]=top->name[j];
            }
            if((ListPos+i)==ListSel) {
                RefreshText(display,win,gc,Colour,30,30+(20*i),top->name,-1,0);
            } else {
                RefreshText(display,win,gc,Colour,30,30+(20*i),top->name,0,0);
            }
            top=top->Next;
		    //fprintf(stdout,"Good line\n");
        }
    } else {
		i++;
	}
	while(i<12) {
        RefreshText(display,win,gc,Colour,30,30+(20*i),"",0,0);
		//fprintf(stdout,"Blank line\n");
		i++;
	}
    //fprintf(stdout,"Refreshed\n");
    if(top==NULL) {
        return(1);
    }
    return(0);
}

struct DirList *
ReadDir2List(char *dirname,char *filter)
{
	struct DirList *MyDirList=NULL;
	DIR *MyDir;
	struct dirent *MyDent;
    MyDir=opendir(dirname);
    if(MyDir) {
        MyDirList=MakeDirList();
		if(!MyDirList) {
			fprintf(stdout,"Going down\n");
		}
        MyDent=readdir(MyDir);
        while(MyDent) {
            AddEntry(MyDirList,MyDent,dirname,filter);
            MyDent=readdir(MyDir);
        }
    }
	return(MyDirList);
}

int 
FindDotDot(char *name)
{
	int i=0;
	int len=strlen(name);
	while(i<len-3) {
		if(name[i]=='/' && name[i+1]=='.' && name[i+2]=='.' && name[i+3]=='/') {
			if(i==0) {
				strcpy(name,&name[3]);
			}
			return(i);
		}
		i++;
	}
	return(0);
}

void 
ClearDot(char *name)
{
	int i=0,len=strlen(name);
	while(i<len-2) {
		if(name[i]=='/' && name[i+1]=='.' && name[i+2]=='/') {
			fprintf(stdout,"murn\n");
			strcpy(&name[i],&name[i+2]);
			return;
		}
		i++;
	}
}
		
void 
FixDirName(char *name)
{
//	while there is a "/../"
//	change "/xxx/../" to "/"
	int i,k,j,done;
	ClearDot(name);
	if((k=FindDotDot(name))) {
		j=k--;
		while(k>-1 && name[k]!='/') {
			k--;
		}
		j+=3;
		for(i=j;name[i];i++) {
			name[k++]=name[i];
		}
		name[k]=0;
	}
//  Change any //s to a /
	done=0;
	while(!done) {
		done=1;
		i=0;
		while(name[i] && name[i+1]) {
			if(name[i]=='/' && name[i+1]=='/') {
				strcpy(&name[i+1],&name[i+2]);
				done=0;
				i--;
			}
			i++;
		}
	}
}

struct DirList *
ReadNewDir(Display *display,Window win,GC gc,int BasePix,struct DirList *dirlist,
		   char textbox[2][100],char *filter)
{
	int AtEndofList=0;//   BUG XXXXXXXXXXXXXXXXXXXXXXXXXXX
	FreeDirList(dirlist);
	FixDirName(textbox[0]);
   	dirlist=ReadDir2List(textbox[0],filter);
	if(dirlist) {
   		if(dirlist->First) {
    		strcpy(textbox[1],dirlist->First->name);
    	}
	}
    AtEndofList=RefreshList(display,win,gc,dirlist,0, BasePix,1);
    RefreshText(display,win,gc,BasePix,30,300,textbox[0],0,0);
    RefreshText(display,win,gc,BasePix,30,350,textbox[1],0,1);
	return(dirlist);
}

void
MessageBox(Display *display,Window win,GC gc,char *mesg,int BasePix)
{
	Window mbox;
	struct Button Okay;
	int done=0;
	XEvent report;
	mbox=XCreateSimpleWindow(display,win,300,100,300,300,2,2,
							 BasePix+MID_GRAY);
	AddButton(display,&Okay,100,245,100,32,"data/buttons/okayu.pcx",17,BasePix,
			  mbox,gc,0);
	XMapWindow(display,mbox);
	DrawMessageStuff(display,mbox,gc,BasePix);
	XDrawImageString(display,mbox,gc,30,100,mesg,strlen(mesg));
	XSync(display,False);
	while(!done) {
		XNextEvent(display,&report);
		if(report.type==ButtonPress || report.type==ButtonRelease) {
			if(report.xany.window==Okay.ButtonWin) {
				if(ButtonReact(display,gc,&report,&Okay)) {
					done=1;
				}
			}
		}
	}
	XUnmapWindow(display,mbox);
	XDestroyWindow(display,mbox);
}

int 
RequestFilename(Display *display,Window win,GC gc,char *buff,
                     char *mesg,int Exists,int PaletteOffset,char *filter,char load)
{
	char tmpstr[100];
	struct DListNode *tmpptr;
    Window Requester;
    XEvent report;
    int closed=0;
    struct Button SaveCancel[2];
    int activeTB=2;
    int AtEndofList=1;
    int Curs=0;
    char letter,ret=0;
    char textbox[2][100] = { "/users/ca4/kbresl/Project/", "" };
    int ListPos=0,ListSel=1;
    struct DirList *MyDirList;
    Requester=XCreateSimpleWindow(display,win,300,100,300,470,2,2,
									PaletteOffset+MID_GRAY);
   	if (load) {  
        AddButton(display,&SaveCancel[0],25,415,100,32, "data/buttons/loadu.pcx",17, PaletteOffset,Requester,gc,0); 
    } else {
        AddButton(display,&SaveCancel[0],25,415,100,32, "data/buttons/saveu.pcx",17, PaletteOffset,Requester,gc,0);
    }
 
    AddButton(display,&SaveCancel[1],175,415,100,32, "data/buttons/cancelu.pcx",19, PaletteOffset,Requester,gc,0);
    XMapWindow(display,Requester);
    XSelectInput(display,Requester,KeyPressMask);
    DrawRequesterStuff(display,Requester,gc,PaletteOffset);
	MyDirList=ReadDir2List(textbox[0],filter);
	if(MyDirList->First) {
		strcpy(textbox[1],MyDirList->First->name);
	}
    RefreshText(display,Requester,gc,PaletteOffset,30,300,textbox[0], 0,0);
    RefreshText(display,Requester,gc,PaletteOffset,30,350,textbox[1], 0,1);
   	AtEndofList=RefreshList(display,Requester,gc,MyDirList,0,
							PaletteOffset,ListSel);
    while(!closed) {
        XNextEvent(display,&report);
        switch(report.type) {
            case KeyPress:
                letter=KeyCode2Ascii(display,report.xkey.keycode,
									report.xkey.state);
                if(letter!=-1) {
                    if(activeTB) {
                        AffectString(textbox[activeTB-1],letter,&Curs);
                        RefreshText(display,Requester,gc,PaletteOffset,30, 300,textbox[0], Curs,activeTB==1);
                        RefreshText(display,Requester,gc,PaletteOffset,30, 350,textbox[1], Curs,activeTB==2);
                    }
                } else {
                    switch(report.xkey.keycode) {
                        case KEY_LEFT:
                            if(activeTB) {
                                if(Curs) Curs--;
                                RefreshText(display,Requester,gc,PaletteOffset, 30,300, textbox[0], Curs,activeTB==1);
                                RefreshText(display,Requester,gc,PaletteOffset, 30,350, textbox[1], Curs,activeTB==2);
                            }
                            break;
                        case KEY_RIGHT:
                            if(activeTB) {
                                if(Curs<strlen(textbox[activeTB-1])) {
                                    Curs++;
                                    RefreshText(display,Requester,gc, PaletteOffset,30, 300,textbox[0], Curs,activeTB==1);
                                    RefreshText(display,Requester,gc, PaletteOffset,30, 350,textbox[1], Curs,activeTB==2);
                                }
                            }
                            break;
                        case KEY_RETURN:
                            if(activeTB==1) { // Re-read dir.
    							FreeDirList(MyDirList);
								if(textbox[0][strlen(textbox[0])-1]!='/') {
									strcat(textbox[0],"/");
								}
								MyDirList=ReadNewDir(display,Requester,gc,PaletteOffset,MyDirList,textbox,filter);
								ListSel=1;
								ListPos=0;
								activeTB=2;
								Curs=0;
                            } else if(activeTB==2) { // Save file
								// Unless that's a dir !!
								if(textbox[1][0]==0) {
									fprintf(stdout,"No string\n");
								} else if(textbox[1][strlen(textbox[1])-1]=='/') {
									strcat(textbox[0],textbox[1]);
									textbox[1][0]=0;
									MyDirList=ReadNewDir(display,Requester,gc,PaletteOffset,MyDirList,textbox,filter);
									ListSel=1;
									ListPos=0;
									activeTB=2;
									Curs=0;
								} else {
									if(IsDir(textbox[0],textbox[1])) {
										if(textbox[0][strlen(textbox[0])-1]!='/') {
											strcat(textbox[0],"/");
										}
										strcat(textbox[0],textbox[1]);
										strcat(textbox[0],"/");
										textbox[1][0]=0;
                                    	MyDirList=ReadNewDir(display,Requester,gc,PaletteOffset,MyDirList,
															textbox,filter);
										ListSel=1;
										ListPos=0;
										activeTB=2;
										Curs=0;
									} else {
										closed=1;
										ret=1;
									}
								}
                            }
                            break;
                        case KEY_UP:
                            if(ListSel>1) ListSel--;
                            if(ListSel<=ListPos) ListPos--;
                            activeTB=2;
                            tmpptr=GetPtr(MyDirList,ListSel-1);
							if(tmpptr) {
                            	strcpy(textbox[1],tmpptr->name);
							} else {
								strcpy(textbox[1],"");
							}
                            RefreshText(display,Requester,gc,PaletteOffset,30,350,textbox[1], Curs=0,1);
                            AtEndofList=RefreshList(display,Requester,gc,MyDirList,ListPos, PaletteOffset,ListSel);
                            break;
                        case KEY_DOWN:
                            if(MyDirList->Length>ListSel) ListSel++;
                            activeTB=2;
                            tmpptr=GetPtr(MyDirList,ListSel-1);
							if(tmpptr) {
                            	strcpy(textbox[1],tmpptr->name);
							} else {
								strcpy(textbox[1],"");
							}
                            RefreshText(display,Requester,gc,PaletteOffset,30,350,textbox[1], Curs=0,1);
                            if(!AtEndofList) {
                                if(ListSel>=ListPos+12) {
                                    ListPos++;
                                }
                            }
                            AtEndofList=RefreshList(display,Requester,gc,MyDirList,ListPos, PaletteOffset,ListSel);
                            break;
                        case KEY_ESC:
                            fprintf(stdout,"Canceled...\n");
                            closed=1;
                            ret=0;  // Don't save...
                            break;
                        default:
                            fprintf(stdout,"Keys was... %d\n",report.xkey.keycode);
                            break;
                    }
                }
                break;
            case ButtonRelease: // Should check save/cancel/textbox ONLY
            case ButtonPress: // Should check save/cancel/textbox ONLY
                if(report.xany.window== SaveCancel[0].ButtonWin) {
                    fprintf(stdout,"Save...\n");
                    if(ButtonReact(display,gc,&report,&SaveCancel[0])) {
                        closed=1;
                        ret=1;  // DO save
                    }
                } else if(report.xany.window== SaveCancel[1].ButtonWin) {
                    fprintf(stdout,"Canceled...\n");
                    if(ButtonReact(display,gc,&report,&SaveCancel[1])) {
                        closed=1;
                        ret=0;  // Don't save...
                    }
                } else {
                    if(InBox(report.xbutton.x-300,report.xbutton.y-100,30,300,269,319)) {
                        activeTB=1;
                        Curs=((report.xbutton.x-300)-40)/8;
						if(Curs>strlen(textbox[0])) {
							Curs=strlen(textbox[0]);
						}
                    } else if(InBox(report.xbutton.x-300,report.xbutton.y-100,30,350,269,369)) {
                        activeTB=2;
                        Curs=((report.xbutton.x-300)-40)/8;
						if(Curs>strlen(textbox[1])) {
							Curs=strlen(textbox[1]);
						}
                    } else if(InBox(report.xbutton.x-300,report.xbutton.y-100,30,50,270,250)) {
                        activeTB=2;
                        ListSel=(report.xbutton.y-134)/20+ListPos;
						if(ListSel==ListPos) { 
							ListSel++;
						}
                        tmpptr=GetPtr(MyDirList,ListSel-1);
						if(tmpptr) {
                           	strcpy(tmpstr,tmpptr->name);
						} else {
							strcpy(tmpstr,"");
						}
						if(tmpstr[strlen(tmpstr)-1]=='/') {
							strcat(textbox[0],tmpstr);
							MyDirList=ReadNewDir(display,Requester,gc,PaletteOffset,MyDirList,textbox,filter);
							textbox[1][0]=0;
							ListSel=1;
                            ListPos=0;
							activeTB=2;
							Curs=0;
						} else {
							strcpy(textbox[activeTB-1],tmpstr);
						}
						Curs=0;
                        AtEndofList=RefreshList(display,Requester,gc,MyDirList,ListPos, PaletteOffset,ListSel);
                    }
                    RefreshText(display,Requester,gc,PaletteOffset,30,300,textbox[0],Curs,activeTB==1);
                    RefreshText(display,Requester,gc,PaletteOffset,30,350,textbox[1],Curs,activeTB==2);
                }
                break;
        }
    }
    FreeDirList(MyDirList);
    // Free Images ????? XXXX
    KillWindow(display,SaveCancel[0].ButtonWin);
    KillWindow(display,SaveCancel[1].ButtonWin);
    KillWindow(display,Requester);
	buff[0]=0;
	strcpy(buff,textbox[0]);
	if(buff[strlen(buff)-1]!='/') {
		strcat(buff,"/");
	}
	strcat(buff,textbox[1]);
	fprintf(stdout,"Selected file is: %s\n",buff);
    return(ret);
}




