#include "palette.h"
#include "mymem.h"
#include "help.h"
#include <X11/Xlib.h>
#include "gui.h"
#include <stdio.h>
#include "ed2.h"
#include "pcx.h"

void 
ShowData(Display *display,Window win,GC gc,char *data,int BasePix) 
{
	char linedata[20][100] = { { 32 } } ; // Space (invisible)
	char *buff;
	int which=0;
	int i;
	int lines=1;
	while(lines) {
		buff=linedata[which];
		while(*data!=0 && *data!='\n') {
			*(buff++)=*(data++);
		}
		if(*data==0) {
			lines=0;
		}
		data++;
		which++;
	}
	XSetForeground(display,gc,BasePix);
	XSetBackground(display,gc,BasePix+REQ_BACK);
	for(i=0;i<20;i++) {
		XDrawImageString(display,win,gc,35,120+(i*14),linedata[i],60);
	}
}

char *
ReadHelpData(char *helpfile)
{	
	FILE *fp;
	int size;
	char *data;
	fp=fopen(helpfile,"rt");
	if(!fp) {
		return(NULL);
	}
	fseek(fp,0,SEEK_END);
	size=ftell(fp)+1;
	fseek(fp,0,SEEK_SET);
	data=MyMalloc(size,"help-text");
	if(data) {
		fread(data,1,size-1,fp);
		data[size-1]=0;
	}
	fclose(fp);
	return(data);
}

char *
LoadData(Window win,struct Button buts[],struct ControlInfo *Controls,Pixmap *icon)
{	
	int i;
	char *data;
	char helpfile[100];
	char type;
	for(i=0;i<40;i++) {
		if(win==buts[i+1].ButtonWin) {
			*icon=buts[i+1].UpPix;
			if(Controls->MotionEditor) {
				type='m';
			} else {
				type='c';
			}
			sprintf(helpfile,"data/help/%ced%02d",type,i);
			data=ReadHelpData(helpfile);
			return(data);
		}
	}
	return(NULL);
}

void
GetHelp(Display *display,Window Parent,GC gc,struct ControlInfo *Controls,
		struct Button Buttons[],int BasePix)
{
	XImage *tmpImg;
	int Helped=0,beep;
	char *data=NULL;
	Pixmap iconpix;
	XEvent report;
	Window HelpWin;
	struct Button OkayButton,CredButton;
	HelpWin=XCreateSimpleWindow(display,Parent,300,100,550,500,2,2,BasePix+MID_GRAY);
	XMapWindow(display,HelpWin);
    Draw3DBox1(display,HelpWin,gc,0,0,550,500,BasePix+DARK_GRAY,BasePix+WHITE,1);
    Draw3DBox1(display,HelpWin,gc,1,1,69,69,BasePix+DARK_GRAY,BasePix+WHITE,1);
    Draw3DBox1(display,HelpWin,gc,70,1,410,69,BasePix+DARK_GRAY,BasePix+WHITE,1);
    Draw3DBox1(display,HelpWin,gc,480,1,69,69,BasePix+DARK_GRAY,BasePix+WHITE,1);
    Draw3DBox1(display,HelpWin,gc,1,70,548,359,BasePix+DARK_GRAY,BasePix+WHITE,1);
    Draw3DBox1(display,HelpWin,gc,1,430,548,69,BasePix+DARK_GRAY,BasePix+WHITE,1);
	/* Pink box  */
    Draw3DBox1(display,HelpWin,gc,25,100,500,300,BasePix+DARK_GRAY,BasePix+WHITE,0);
    XSetForeground(display,gc,BasePix+REQ_BACK);
    XFillRectangle(display,HelpWin,gc,26,101,498,298);
	/* Title */
	tmpImg=WLoad_PCX("data/buttons/helphead.pcx",0,display,BasePix);
	if(tmpImg) {
		XPutImage(display,HelpWin,gc,tmpImg,0,0,143,10,tmpImg->width,
				  tmpImg->height);
	}

    tmpImg=WLoad_PCX("data/buttons/helpicon.pcx",0,display,BasePix);
	if(tmpImg) {
		XPutImage(display,HelpWin,gc,tmpImg,0,0,20,20,32,32);
		XPutImage(display,HelpWin,gc,tmpImg,0,0,498,20,32,32);
	}
    WFreePic(tmpImg);

	AddButton(display,&OkayButton,100,450,100,32,"data/buttons/okayu.pcx",17,BasePix,HelpWin,gc,0);
	AddButton(display,&CredButton,350,450,100,32,"data/buttons/credu.pcx",17,BasePix,HelpWin,gc,0);

	data=ReadHelpData("data/help/message");
	if(data) {
		ShowData(display,HelpWin,gc,data,BasePix);
	}

	XSync(display,False);
	while(!Helped) {
		if(XCheckMaskEvent(display,ButtonReleaseMask|ButtonPressMask|KeyPressMask,&report)) {
			beep=1;
			switch(report.type) {
				case ButtonPress:
					if(data) {
						MyFree(data,"help-data");
					}
					data=LoadData(report.xany.window,Buttons,Controls,&iconpix);
					if(data) {
						if(iconpix) {
							XCopyArea(display,iconpix,HelpWin,gc,0,0,32,32,20,20);
							XCopyArea(display,iconpix,HelpWin,gc,0,0,32,32,498,20);
						} else {
							fprintf(stdout,"no iconpix\n");
						}
						ShowData(display,HelpWin,gc,data,BasePix);
						beep=0;
					} 
					/* There is NO break here... below is for both Press/Release */
				case ButtonRelease: 
				    if(report.xany.window==OkayButton.ButtonWin) {
							if(ButtonReact(display,gc,&report,&OkayButton)) {
							Helped=1;
						}
						beep=0;
					} 
				    if(report.xany.window==CredButton.ButtonWin) {
						if(ButtonReact(display,gc,&report,&CredButton)) {
							if(data) {
								MyFree(data,"help-data");
							}
							data=ReadHelpData("data/help/credits");
							if(data) {
								ShowData(display,HelpWin,gc,data,BasePix);
							}
						}
						beep=0;
					} 
					if(beep && report.type==ButtonPress) {
						XBell(display,ERROR_VOLUME);
					}
					break;
				case KeyPress:
					Helped=1;
					break;
			}
		}
	}
	XUnmapWindow(display,HelpWin);
	XDestroyWindow(display,HelpWin);
	if(data) {
		MyFree(data,"help-data");
	}
}

