 
 
/***********************************************************************
 *
 *      WRITEDDC.C      WRITEDDC.C
 *
 *  This file is contatins functions used to write Character data to
 *  a ".ddc" file.
 *  Only one of the functions is public: WriteCharacter()
 *
 **********************************************************************/
 
 
#include <stdio.h>
#include "writeddc.h"
#include "mymem.h"
#include "generic.h"

 
/*********************************************************************
 *
 *	WriteDDCHead()									WriteDDCHead()
 *
 *	Writes a DDC header (signiture+version number) to the open
 *	file.
 *
 */

static void 
WriteDDCHead(FILE *fp,int version)
{
    fputc('D',fp);
    fputc('D',fp);
    fputc('C',fp);
    fputc(version,fp);
} 

/*********************************************************************
 *
 *	WriteObjects()									WriteObjects()
 *	
 *	Writes object data to the file, the objects are stored in a
 * 	null terminated array of pointers. Data is written until the
 * 	null pointer is reached.
 * 
 */
 
static void 
WriteObjects(FILE *fp,int Num,Segment **objes)
{
    int i,j,bollix;
    struct FILEobject *Fobjs,*tmp;
    Segment *thisone;
    Fobjs=MyMalloc(sizeof(struct FILEobject)*Num,"FileObjs");
    if(Fobjs) {
    	tmp=Fobjs;
    	for(i=0;i<Num;i++) {
        	thisone=GetObj(i,objes);
        	bollix=thisone->Numpoints;
        	tmp->Numpoints=bollix;
        	tmp->Poff=thisone->Poffset;
        	if(thisone->Prev) {
        		tmp->PrevObj=thisone->Prev->ID;
        	} else {
        		tmp->PrevObj=-1;
        	}
        	for(j=0;j<5;j++) {
        		if(thisone->Next[j]) {
            		tmp->Next[j]=thisone->Next[j]->ID;
        		} else {
            		tmp->Next[j]=-1;
        		}
        	}
        	for(j=0;j<20;j++) {
        		tmp->Name[j]=0x99;
        	}
        	tmp++;
    	}
    	fwrite(Fobjs,sizeof(struct FILEobject),Num,fp);
    	MyFree(Fobjs,"FileObjs");
    }
}

/*********************************************************************
 *
 *	WritePolygons()									WritePolygons()
 *	
 *	Writes polygon data to the file, the polygons are stored in a
 * 	null terminated array of pointers. Data is written until the
 * 	null pointer is reached.
 * 
 */
 
static void 
WritePolygons(FILE *fp,int Num,Polygon **plys)
{
    int i,j;
    struct FILEpolygon *Fpolys,*tmp;
    Fpolys=MyMalloc(sizeof(struct FILEpolygon)*Num,"FilePolys");
    if(Fpolys) {
    	tmp=Fpolys;
    	for(i=0;i<Num;i++) {
        	for(j=0;j<4;j++) {
        		tmp->Points[j]=plys[i]->Points[j];
        	}
        	tmp->Colour=plys[i]->Color;
        	tmp->Texture=plys[i]->Texture;
        	tmp->CentrePt=plys[i]->CentrePt;
        	tmp->Style=plys[i]->Style;
        	tmp->reserved=0;
        	tmp++;
    	}
    	fwrite(Fpolys,sizeof(struct FILEpolygon),Num,fp);
    	MyFree(Fpolys,"FilePolys");
    }
}

/********************************************************************
 *
 *	WritePoints()									WritePoints()
 *
 *	Writes the points data to the file. Points are written in
 *	x,y,z triples. 
 *
 */
 
static void 
WritePoints(FILE *fp,int Num,short *ptsX,short *ptsY,short *ptsZ)
{
    int i;
    for(i=0;i<Num;i++) {
		fwrite(ptsX++,2,1,fp);
		fwrite(ptsY++,2,1,fp);
		fwrite(ptsZ++,2,1,fp);
    }
}

/*******************************************************************
 *
 *	WriteColls()									WriteColls()
 *
 *	Writes the list of points which are to be checked for collisions.
 *	This is called twice for each character to be written, once for
 *  poits which s/he can hit with, and once for points which can be
 *	hit.
 *
 */
 
static void 
WriteColls(FILE *fp,int num,int pts[])
{
    int i;
    for(i=0;i<num;i++) {
        fwrite(&pts[i],4,1,fp);
    }
}

/********************************************************************
 *
 *	WriteFloorPts()									WriteFloorPts()
 *
 * 	Floor points are the points that generaly stay on the floor.
 *	They dictate the vertical position of the character, there is
 *	one for each foot.
 *
 */

static void
WriteFloorPts(FILE *fp,Dungeon *dung)
{
	fputshort(dung->feet[0][0],fp);
	fputshort(dung->feet[0][1],fp);
}

/*****************************************************************
 *
 *	WriteCharacter()								WriteCharacter()
 *
 *	Writes a characters data for a give filename.  This information
 *	for the character is extracted from the "dungeon" structure, using
 * 	the supporting functions in this file.
 *
 */ 

void
WriteCharacter(Dungeon *dung,char *fname)
{
    FILE *fp;
    fp=fopen(fname,"wb");
    if(fp) {
        WriteDDCHead(fp,DDC_VERSION);
        fputshort(dung->NumObjects,fp);
        fputshort(dung->NumPolygons,fp);
        fputshort(dung->NumPoints,fp);
        fputshort(dung->NumColls[0],fp);  /* Number of Collision points */
        fputshort(dung->NumHits[0],fp);  /* Number of Hit points */
        printf("Wrote Header (v %d)\n",DDC_VERSION);
        WriteObjects(fp,dung->NumObjects,dung->ObjectArray);
        WritePolygons(fp,dung->NumPolygons,dung->PolygonArray);
        WritePoints(fp,dung->NumPoints,dung->Points[0],dung->Points[1],
					dung->Points[2]);
        WriteColls(fp,dung->NumColls[0],dung->Colls[0]);
        WriteColls(fp,dung->NumHits[0],dung->Hits[0]);
		WriteFloorPts(fp,dung);
        fclose(fp);
    }
}
 

