

/***********************************************************************
 *
 *		WRITEDDM.C		WRITEDDM.C
 *
 *  This file is contatins functions used to write Movement data to
 *	a ".ddm" file.
 *	Only one of the functions is public: WriteMoves()
 *
 **********************************************************************/


#include <stdio.h>
#include "ddm.h"
#include "mymem.h"
//#include "../moves.c"

/**********************************************************************
 *
 *	GetMoveSize()									GetMoveSize()
 *
 *	Figures how much buffer space is required for the data for one 
 *	Segment.
 *
 */

static int
GetMoveSize(struct movement *move)
{
	int size=0;
	if(move) {
		size+=sizeof(struct FileMove);
		size+=GetMoveSize(move->Moves);
	}
	return(size);
}

/**********************************************************************
 *
 *	GetBigMoveSize()								GetMoveSize()
 *
 *	Figures out how much buffer space is required for a given
 *	movement.
 *
 */

static int 
GetBigMoveSize(struct Bigmove *move)
{
	int size=0;
	if(move) {
		size+=sizeof(struct FileBigMove);
		size+=GetMoveSize(move->Move);
		size+=GetBigMoveSize(move->Next);
	}
	return(size);
}

/********************************************************************
 *
 *	MoveToBuffer()								MoveToBuffer()
 *	
 *	Copies the movement data for one segment for one move to the
 *	buffer to be saved.
 *	Returns the address of the next free position in the buffer.
 *
 */

static int *
MoveToBuffer(struct movement *Move,int *Buff)
{
	struct FileMove *move;
	move=(struct FileMove *)Buff;
	move->DestX=Move->DestXAng;
	move->DestY=Move->DestYAng;
	move->DestZ=Move->DestZAng;
	move->DeltX=Move->DeltXAng;
	move->DeltY=Move->DeltYAng;
	move->DeltZ=Move->DeltZAng;
	move->Waiter=Move->Waiter;
	move->MotX=Move->MotionX;
	move->MotY=Move->MotionY;
	move->MotZ=Move->MotionZ;
	if(Move->Moves) {
		move->NotLast=1;
	}
	Buff+=(sizeof(struct FileMove)>>2);
	if(Move->Moves) {
		Buff=MoveToBuffer(Move->Moves,Buff);
	}
	return(Buff);
}

/****************************************************************
 *
 *	BigMoveToBuffer()							BigMoveToBuffer()
 *
 *	Copies Data for a whole movement into the supplied buffer.
 *	Returns the address of the next free position in the buffer.
 *
 */

static int *
BigMoveToBuffer(struct Bigmove *Move,int *Buff)
{
	struct FileBigMove *BMove;
	BMove=(struct FileBigMove *)Buff;
	BMove->ObjectId=Move->ObjectID;
	if(Move->Next) {
		BMove->NotLast=1;
	}
	Buff+=(sizeof(struct FileBigMove)>>2);
	Buff=MoveToBuffer(Move->Move,Buff);
	if(Move->Next) {
		Buff=BigMoveToBuffer(Move->Next,Buff);
	}
	return(Buff);
}

/****************************************************************
 *
 *	WriteDdmData()								WriteDdmData()
 *
 *	Writes the header and data buffer to the selected file.
 *
 */

static void
WriteDdmData(char *name,struct ddmheader *Header,int *MovePtrs,int byte_size1,
			 int *Buff,int byte_size2)
{
	FILE *fp;
	fp=fopen(name,"wb");
	if(fp) {
		fwrite(Header,sizeof(struct ddmheader),1,fp);
		fwrite(Buff,byte_size2,1,fp);
		fclose(fp);
	}
}

/******************************************************************
 *
 *	WriteMoves()								WriteMoves()
 *
 *	Writes the null terminated array of moves to the file specified.
 *	Using the supporting fuctions in this file, it converts the
 *	link-lists into a writable format.
 *
 */

int 
WriteMoves(char *name,struct Bigmove *Moves[])
{
	int Number;
	int i,size=0,success=0;
	struct ddmheader Header;
	int *MovePtrs;
	int *Buff,*tmpPtr;

	/* Number of Moves in File */
	for(Number=0;Moves[Number];Number++);

	/* Fill in header */
	Header.Identity[0]='D';
	Header.Identity[1]='D';
	Header.Identity[2]='M';
	Header.Identity[3]=0;
	Header.NumMoves=Number;
	for(i=0;i<4;i++) {
		Header.Reserved[i]=0;
	}

	/* Size of data */
	for(i=0;i<Number;i++) {
		size+=GetBigMoveSize(Moves[i]);
	}
	MovePtrs=MyMalloc(4*Number,"writeddm:moveptrs");
	if(MovePtrs) {
		Buff=MyMalloc(size,"MovesBuffer");
		if(Buff) {
			tmpPtr=Buff;
			for(i=0;i<size>>2;i++) {
				*(tmpPtr++)=0;
			}
			tmpPtr=Buff;
			for(i=0;i<Number;i++) {
				/* Lets Go... */
				MovePtrs[i]=(int) (tmpPtr-Buff);
				tmpPtr=BigMoveToBuffer(Moves[i],tmpPtr);
			}
			WriteDdmData(name,&Header,MovePtrs,Number*4,Buff,size);
			success=1;
			MyFree(Buff,"MovesBuffer");
		}
		MyFree(MovePtrs,"MovePtrs");
	}
	return(success);
}


