#include "poly.h"
#include "mymem.h"
#include "graph.h"
#include "id.h"
#include "pcx.h"
#include "generic.h"
#include "window.h"
#include "misc.h"

void
HLine(char *rast,int x,int y,int len,int colour)
{
    rast+=(y*MAX_WINDOW_WIDTH)+x;
    while(len--) {
        *(rast++)=colour;
    }
}
 
void
VLine(char *rast,int x,int y,int len,int colour)
{
    rast+=(y*MAX_WINDOW_WIDTH)+x;
    while(len--) {
        *rast=colour;
        rast+=MAX_WINDOW_WIDTH;
    }
}
 
void
DrawVerticalLine(uchar *dest,int sx, int sy,int length,char col)
{
	int offset=0;
	offset=(MAX_WINDOW_WIDTH*sy)+sx;
	while(length--) {
		*(dest+offset)=col;
		offset+= MAX_WINDOW_WIDTH;
	}	
}

void
DrawVerticalTexturedLine(uchar *dest,int sx, int sy,int length,
						struct Picture_Type *wall, int textoffset,int basepix)
{
	int error=0,y_term=0;

	dest+=(MAX_WINDOW_WIDTH*sy)+sx;
	y_term=(wall->Height<<8)/length;
	while(length--) {
		error+=y_term;
		if (*(wall->Data+textoffset)!=(basepix+131)) {
			*(dest)=*(wall->Data+textoffset);
		}
		dest+= MAX_WINDOW_WIDTH;
		while(error >256) {
			textoffset+=wall->Width;
			error-=256;
		}
	}	
}

void
XXXXXXXSetupWalls(int vals[])
{
	int x=0,i,y=1500;
	long d=-1500; 
 
	for( i=0;i<(MAX_WINDOW_WIDTH/2);i++) {
		if( d< 0) { 
 			d+=(2*x+3);
			x++;
		} else {
			d+=(2*(x-y)+5);
			y--;
			x++;
		}
		vals[i] = y-1300;
	}
}

void
XXXXXXXDoWalls(uchar *dest,struct Picture_Type *wall,int offset,int yoffset,int size,int vals[],int basepix)
{ 
	int i,y1;
	offset<<=4;
	//tmp=wall->Width*100;
	fprintf(stdout,"vals[0] %d, yoffset %d\n",vals[0],yoffset);

	for(i=0;i<(MAX_WINDOW_WIDTH/2);i++)
	{ 
		y1=440-(vals[i]<<1)+(38-yoffset);	
		DrawVerticalTexturedLine(dest,MAX_WINDOW_WIDTH/2+i,vals[i],y1,wall,(i-offset) & 511,basepix); 
										//(tmp+i-offset) % wall->Width ); // dest,xpos,ypos,size,col
		DrawVerticalTexturedLine(dest,MAX_WINDOW_WIDTH/2-i,vals[i],y1,wall,(-offset-i) & 511,basepix); 
										// (tmp-offset-i) % wall->Width);
	}
}

void
ScaleBitmap(XImage *dest,struct Picture_Type *src,int destx,int desty,int size,int basepix)
{
	int i=0,j=0;
	int width=src->Width<<6;
	int xinc=width/size; 	
	int ysize=(src->Height*size)>>8;///4;	

	for(i=0;i < width;i+=xinc,j++) {
		DrawVerticalTexturedLine(dest->data,destx+j,
								desty,ysize,src,i>>6,basepix);
	}
}

void 
DoFight(XImage *dest,struct Picture_Type *src,int destx,int desty,int size,int basepix)
{
	ScaleBitmap(dest,src,destx-(size/2),desty,size,basepix);
}

void 
DrawBang(XImage *back,struct bang *bang,struct Picture_Type **bangpic)
{
	if(bang->Show) {
		CopyAreaTrans(bangpic[bang->What]->Data,0,0,bangpic[bang->What]->Width,
					back->data, bang->X+512-50,bang->Y+284-50,1024,100,100,48);
	}
}

void
DrawBangs(XImage *back,struct bang *bang1,struct bang *bang2,
		 struct Picture_Type **bangpics)
{
	DrawBang(back,bang1,bangpics);
	DrawBang(back,bang2,bangpics);
}

void
SetPaletteFromFile(Display *display,Colormap ColMap,int BasePix,char *pcxname)
{
    struct Picture_Type *pal;
    pal=Load_PCX(pcxname,1);
    if(pal) {
        SetPalette3(display,ColMap,pal->Palette,BasePix,192);
        FreePic(pal);
    }
}
 
void
GetPaletteFromFile(char *pcxname,char *buff)
{
    struct Picture_Type *pal;
    int i;
    pal=Load_PCX(pcxname,1);
    if(pal) {
        for(i=0;i<768;i++) {
            buff[i]=pal->Palette[i];
        }
        FreePic(pal);
    }
}
 
void
FadeToBlack(Display *display,Colormap ColMap,char pal[],int BasePix,int delay)
{
    int i,j;
    for(i=0;i<256;i++) {
        for(j=0;j<768;j++) {
            if(pal[j]) {
                pal[j]--;
            }
        }
        SetPalette3(display,ColMap,pal,BasePix,192);
        //XSync(display,False); // I think this was the bug.
								// The one that stops input events.
        WaitAWhile(delay);
    }
}
 
void 
RemapPicture(struct Picture_Type *Text, int AddOn, int Num)
{
    int i, j;
    int biggest = 0;
    uchar *tmp;
    tmp = Text->Data;
    for(i = 0; i < Text->Width; i++) {
		for(j = 0; j < Text->Height; j++) {
	    	if(*tmp > biggest) {
				biggest = *tmp;
			    if(biggest > 191) {
					//fprintf(stderr, "Texture %d: Biggest: %d\n", Num, biggest);
    			}
			}
	    	*(tmp++) += AddOn;
		}
    }

    if(biggest > 191) {
		//fprintf(stderr, "Texture %d: Biggest: %d\n", Num, biggest);
    }
}

void 
RemapTextures(struct Picture_Type **Texts, int AddOn, int Num)
{
    int i;
    for(i = 0; i < Num; i++) {
		RemapPicture(Texts[i], AddOn, i);
    }
}

void 
SetPalette3(Display * display, Colormap Cols, 
		  uchar *pal, int first, int num)
{
    int i;
    uchar *tmp;
    XColor *colours, *tmp2;
    colours = MyMalloc(num * sizeof(XColor), "Xcolors");
    if(colours) {
		tmp2 = colours;
		if(pal) {
	   		tmp = pal;
	   		for(i = first; i < first + num; i++) {
				tmp2->red =(*(tmp++)) << 8;
				tmp2->green =(*(tmp++)) << 8;
				tmp2->blue =(*(tmp++)) << 8;
				tmp2->pixel = i;
				tmp2->flags = DoRed | DoBlue | DoGreen;
				tmp2++;
	   		}
	   		XStoreColors(display, Cols, colours, num);
		} else {
	   		fprintf(stderr, "Can't find the palette\n");
		}
		MyFree(colours, "XColors");
   	} else {
		fprintf(stderr, "no mem\n");
	}
}
#ifdef SWEET

/**************************************************************8888
 */

void 
LoadImageCentre(XImage *scr, int off, char *name,int Width,int Height)
{
    int k, j;
    struct Picture_Type *Title;
    uchar *tmp1, *tmp2;
//	ClearScreenSWEET(scr,0,0,scr->width-1,scr->height-1);
    Title = Load_PCX(name, 0);
    if(Title) {
		RemapPicture(Title, off, -1);
		tmp1 = Title->Data;
		tmp2 =(uchar *)(scr->data);
		tmp2 +=(Width - Title->Width) / 2;
		tmp2 +=((Height - Title->Height) / 2) * Width;
		for(j = 0; j < Title->Height; j++) {
			  for(k = 0; k < Title->Width; k++) {
				  *(tmp2++) = *(tmp1++);
			  }
			  tmp2 +=(Width - Title->Width);
		}
		FreePic(Title);
    } else {
		fprintf(stdout,"Can't Load image: %s\n",name);
    }
}

#endif

void 
PutVPix(XImage * vscreen, int x, int y, uchar col)
{
    *((vscreen->data) + x +(MAX_WINDOW_WIDTH * y)) = col;
}


void 
ClearScreen(XImage * Img)
{
    int i, j;
    int *tmp;
    tmp =(int *)(Img->data);
    for(i = 0; i < MAX_WINDOW_HEIGHT; i++) {
		for(j = 0; j < MAX_WINDOW_WIDTH >> 2; j++) {
	    	*(tmp++) = 0;
			//100+(100<<8)+(100<<16)+(100<<24);	/*(j/2)& 0xFF;  */
		}
    }
}

//#ifndef SWEET
void 
ClearScreenSWEET(XImage * Img, int minx, int miny, int maxx, int maxy)
{
	static int col=1;
	int *tmp;
	int width,height,i,j;
	tmp=(int *)Img->data;
	tmp+=(miny<<8);
	tmp+=(minx>>2);
	height=(maxy-miny)-1;
	width=(maxx-minx+7)>>2;
	for(i=0;i<height;i++) {
		for(j=0;j<width;j++) {
			*(tmp++)=0;
		}
		tmp+=256-width;
	}
	col++;
}
//#endif
void 
linedraw(int x1, int y1, int x2, int y2, int color, uchar * vscreen)
{
    int y_unit, x_unit;
    int offset = y1 * MAX_WINDOW_WIDTH + x1;
    int ydiff = y2 - y1;
    int xdiff;
    int error_term = 0;
    int length;
    int i;
    
    if(ydiff < 0) {
	ydiff = -ydiff;
	y_unit = -MAX_WINDOW_WIDTH;
    } else
	y_unit = MAX_WINDOW_WIDTH;
    
    xdiff = x2 - x1;
    
    if(xdiff < 0) {
	xdiff = -xdiff;
	x_unit = -1;
    } else
	x_unit = 1;
    
    if(xdiff > ydiff) {
	length = xdiff + 1;
	for(i = 0; i < length; i++) {
		if ((offset>MAX_WINDOW_HEIGHT*MAX_WINDOW_WIDTH)||(offset<0)) { 
			return;
		}
	    vscreen[offset] = color;
	    offset += x_unit;
	    error_term += ydiff;
	    if(error_term > xdiff) {
		error_term -= xdiff;
		offset += y_unit;
	    }
	}
    } else {
	length = ydiff + 1;
	for(i = 0; i < length; i++) {
		if ((offset>MAX_WINDOW_HEIGHT*MAX_WINDOW_WIDTH)||(offset<0)) { 
			return;
		}
	    vscreen[offset] = color;
	    offset += y_unit;
	    error_term += xdiff;
	    if(error_term > 0) {
		error_term -= ydiff;
		offset += x_unit;
	    }
	}
    }
}


/**********************************************************************
 *
 *	CopyArea()												CopyArea()
 *
 *	Copy a rectangular area from one buffer to another.
 *  This should be done in asm, it will be used to blast
 *  the background to the vscreen.
 *
 */

void
CopyAreaTrans(uchar *src,int srcx,int srcy,int srcw,
		 uchar *dest,int destx,int desty,int destw,
		 int width,int height,int Trns)
{
	int i,j;
	src+=(srcx+(srcy*srcw));
	dest+=(destx+(desty*destw));
	for(i=0;i<height;i++) {
		for(j=0;j<width;j++) {
			if(*src!=Trns) {
				*(dest++)=*(src++);
			} else {
				dest++;
				src++;
			}
		}
		dest+=(destw-width);
		src+=(srcw-width);
	}
}

void
CopyArea(uchar *src,int srcx,int srcy,int srcw,
		 uchar *dest,int destx,int desty,int destw,
		 int width,int height)
{
	int i,j;
	src+=(srcx+(srcy*srcw));
	dest+=(destx+(desty*destw));
	for(i=0;i<height;i++) {
		for(j=0;j<width;j++) {
			*(dest++)=*(src++);
		}
		dest+=(destw-width);
		src+=(srcw-width);
	}
}


