#include "readddm.h"
#include "pcx.h"
#include "polyinit.h"
#include "mymem.h"
#include "graph.h"
#include "generic.h"
#include "id.h"

#ifdef SWEET
extern int Player2Objs;
#endif

void 
FreeTextures (struct Picture_Type **txts)
{
    int i=0;
 	while(txts[i]) {
		FreePic(txts[i]);
		i++;
    }
    MyFree (txts, "texture ptrs");
}

void 
FreeNewPoints(Dungeon *dung)
{
    MyFree(dung->newpts[0], "PointsX");
    MyFree(dung->newpts[1], "PointsY");
    MyFree(dung->newpts[2], "PointsZ");
}

void 
FreePoints(Dungeon *dung)
{
    MyFree(dung->Points[0], "PointsX");
    MyFree(dung->Points[1], "PointsY");
    MyFree(dung->Points[2], "PointsZ");
}

void 
Free2DPoints(Dungeon *dung)
{
	MyFree(dung->twoDX,"twoDX");
	MyFree(dung->twoDY,"twoDY");
	MyFree(dung->twoDZ,"twoDZ");
}


void 
FreeAllPoints (Dungeon *dung)
{
	FreePoints(dung);
	FreeNewPoints(dung);
#ifndef SWEET
	Free2DPoints(dung);
#endif
}

void 
FreePolygons (Dungeon *dung)
{
    int i=0;
	while(dung->PolygonArray[i]) {
		MyFree(dung->PolygonArray[i], "polygon_array(element)");
		i++;
    }
    MyFree(dung->PolygonArray, "polygon_array");
    MyFree(dung->SortingPolygons, "spolygon_array");
}

void 
FreeObjects(Segment **objs)
{
    int i=0;
	while(objs[i]) {
		MyFree(objs[i++], "object_array(element)");
    }
    MyFree(objs, "object_array");
}

void 
FreeCollisionPts(Dungeon *dung)
{
	MyFree(dung->Colls[0],"Pl1: Colls\n");
	MyFree(dung->Hits[0],"Pl1: Hits\n");
#ifdef SWEET
	MyFree(dung->Colls[1],"Pl2: Colls\n");
	MyFree(dung->Hits[1],"Pl2: Hits\n");
#endif
}


void 
FreeStuff(Dungeon *dung)
{
	FreeCollisionPts(dung);
    FreeObjects(dung->ObjectArray);
    FreePolygons(dung);
    FreeTextures(dung->Textures);
	FreeAllPoints(dung);
}

struct Picture_Type **
LoadTextures(int Number)
{
    int i;
	struct Picture_Type **Textures;
    char tmp[21];
    Textures = (struct Picture_Type **)
	MyMalloc ((Number + 1) * sizeof (struct Picture_Type *), "Texture ptrs");
    if (Textures) {
	for (i = 0; i < Number; i++) {
	    sprintf (tmp, "textures/txt%02d.pcx%c", i, 0);
	    Textures[i] = Load_PCX (tmp, 0);
	    if (!Textures[i])
			break;
	}
	Textures[i]=NULL;
	if (i < Number) {
	    while (i--) {
			FreePic(Textures[i]);
	    }
	    MyFree(Textures, "Texture ptrs");
	    return (NULL);
	}
	return (Textures);
    }
    return (NULL);
}

void 
ReadInSegments (FILE * fp, int Number, Segment **objs, int first,
		     int Pts, int Ply)
{
    int i, j;
    struct FILEobject tmp;
    for (i = first; i < Number + first; i++) {
		fread (&tmp, sizeof (struct FILEobject), 1, fp);
		for (j = 0; j < 5; j++) {
	    	if (tmp.Next[j] == -1) {
				objs[i]->Next[j] = NULL;
	    	} else {
				objs[i]->Next[j] = objs[tmp.Next[j] + first];
	    	}
		}
		if (tmp.PrevObj == -1) {
	    	objs[i]->Prev = NULL;
		} else {
	    	objs[i]->Prev = objs[tmp.PrevObj + first];
		}
		objs[i]->Numpoints = tmp.Numpoints;
		objs[i]->Poffset = tmp.Poff + Pts;

		objs[i]->ID = i;
		objs[i]->CurrXAng = 0;
		objs[i]->CurrYAng = 0;
		objs[i]->CurrZAng = 0;
		objs[i]->Moves = NULL;
		objs[i]->Waiter = 0;
		objs[i]->Returning = 0;
    }
}

Segment **
LoadObjects (FILE * fp1, FILE * fp2, int Num1, int Num2,
							 int Pts1, int Pts2, int Ply1, int Ply2)
{
	Segment **o_array;
    int i, j;
#ifdef SWEET
    Player2Objs = Num1+2;
#endif
    o_array = (Segment **)
	MyMalloc ((Num1 + Num2 + 3) * sizeof (Segment *), "object ptrs");
    if (o_array) {
		for (i = 0; i < Num1 + Num2 + 2; i++) {
			o_array[i] = (Segment *)
				MyMalloc (sizeof (Segment), "object");
			if (!o_array[i])
				break;
		}
		o_array[i]=NULL; /* null terminate the array */
		if (i != Num1 + Num2 + 2) {
			for (j = i; j >= 0; j--) {
				MyFree (o_array[j], "object");
			}
			MyFree (o_array, "object ptrs");
			return (NULL);
		}
		ReadInSegments (fp1, Num1, o_array, 2, Pts1, Ply1);
		ReadInSegments (fp2, Num2, o_array, Num1+2, Pts2, Ply2);
		for(i=0;i<5;i++) {		
			o_array[0]->Next[i]=NULL;
			o_array[1]->Next[i]=NULL;
		}
		o_array[0]->Numpoints=6;
		o_array[0]->Poffset=10;
		o_array[0]->CurrXAng=0;
		o_array[0]->CurrYAng=0;
		o_array[0]->CurrZAng=0;
		o_array[0]->Moves=NULL;
		strcpy(o_array[0]->Name,"Virtual insanity");
		o_array[0]->Waiter=0;
		o_array[0]->Returning=0;

		o_array[1]->Numpoints=6;
		o_array[1]->Poffset=16;
		o_array[1]->CurrXAng=0;
		o_array[1]->CurrYAng=0;
		o_array[1]->CurrZAng=0;
		o_array[1]->Moves=NULL;
		strcpy(o_array[1]->Name,"Virtual insanity");
		o_array[1]->Waiter=0;
		o_array[1]->Returning=0;

		return (o_array);
	}
	return (NULL);
}

void 
LoadPolys (FILE * fp, int Num, Polygon **plys, int first, int Pts)
{
    int i, j;
    struct FILEpolygon tmp;
	//fprintf(stdout,"Loading polys %d -> %d\n",first,first+Num-1);
    for (i = first; i < first + Num; i++) {
		fread (&tmp, sizeof (struct FILEpolygon), 1, fp);
		//plys[i]->ID = 0;
		//plys[i]->NumVerts = tmp.NumVert;
		for (j = 0; j < 4; j++) {
	    	plys[i]->Points[j] = tmp.Points[j] + Pts;
		}

		plys[i]->Color = tmp.Colour ;
		plys[i]->Texture = tmp.Texture;
		plys[i]->Style=tmp.Style;
		if (tmp.CentrePt < -10000) {
	    	plys[i]->CentrePt = 0;
		} else {
	    	plys[i]->CentrePt = tmp.CentrePt + Pts;
		}
    }
}

Polygon *
GenLoadPoly(FILE *fp,int Pts)
{
	Polygon *pol;
	int j;
	struct FILEpolygon tmp;
	fread (&tmp, sizeof (struct FILEpolygon), 1, fp);
	pol=MyMalloc(sizeof(Polygon),"polygon");
	if(pol) {
		for (j = 0; j < 4; j++) {
	    	pol->Points[j] = tmp.Points[j] + Pts;
		}
		pol->Color = tmp.Colour ;
		pol->Texture = tmp.Texture;
		pol->Style=tmp.Style;
		if (tmp.CentrePt < -10000) {
	    	pol->CentrePt = 0;
		} else {
	    	pol->CentrePt = tmp.CentrePt + Pts;
		}
	}
	return(pol);
}

int
GenLoadPolygons(Dungeon *dung,FILE *fp1,FILE *fp2,int Num1,int Num2,
				int Pts1,int Pts2)
{
	Polygon **Parray;
	Polygon **Spol;
	int i,Failed=0;
	int po=0;
	Parray= MyMalloc ((Num1+Num2+1+PLY_MARG)*sizeof(Polygon *),"polygon ptrs");
	if(Parray) {
		Spol=MyMalloc((Num1+Num2+1+1+PLY_MARG)*sizeof(Polygon *),"sorting polygon ptrs");
		if(Spol) {
			for(i=0;i<Num1;i++) {
				Parray[po++]=GenLoadPoly(fp1,Pts1);
			}
			for(i=Num1;i<Num1+Num2;i++) {
				Parray[po++]=GenLoadPoly(fp2,Pts2);
			}
			Parray[po]=NULL;
			for(i=0;i<Num1+Num2;i++) {
				Spol[i]=Parray[i];
				if(!Parray[i]) {
					Failed=1;
				}
			}
			Spol[i]=NULL;
			Spol[i+1]=NULL;
			if(Failed) {
				for(i=0;i<Num1+Num2;i++) {
					if(Parray[i]) {
						MyFree(Parray[i],"polygon");
					}
				}
			} else {
				dung->PolygonArray=Parray;
				dung->SortingPolygons=Spol;
				return(1);
			}
		}
	}
	return(0);
}

void
LoadNewBit(Dungeon *dung,Segment *towhat,char *fname)
{
    FILE *fp;
    short int *tx,*ty,*tz;
    int i,NewOs,NewPls,NewPts,which;
    fprintf(stdout,"Loading (NewBit) \"%s\"\n",fname);
    for(which=0;which<5;which++) {
        if(towhat->Next[which]==NULL)
            break;
    }
    if(which==5) {
        fprintf(stdout,"this object has 5 limbs already\n");
        return;
    }
    fp=fopen(fname,"rb");
    if(fp) {
        if(IsDDC(fp)) {
            fprintf(stderr,"okay: %s\n",fname);
            NewOs=fgetshort(fp);
            NewPls=fgetshort(fp);
            NewPts=fgetshort(fp);
            fseek(fp,4,SEEK_CUR);  /* took out two fgetshort(fp); 's */
            fprintf(stdout,"NewObjects: O:%d, Ply:%d, Pts:%d\n",NewOs,NewPls,
                    NewPts);
            for(i=dung->NumObjects;i<dung->NumObjects+NewOs;i++) {
                dung->ObjectArray[i]=MyMalloc(sizeof(Segment),
                            "NewBit object");
            }
			dung->ObjectArray[i]=NULL;
            ReadInSegments(fp,NewOs,dung->ObjectArray,dung->NumObjects,
                            dung->NumPoints,dung->NumPolygons);
            fprintf(stdout,"Read in objects\n");
            for(i=dung->NumPolygons;i<dung->NumPolygons+NewPls;i++) {
                dung->PolygonArray[i]=MyMalloc(sizeof(Polygon),
                                "NewBit polygon");
                dung->SortingPolygons[i]=dung->PolygonArray[i];
            }
            LoadPolys(fp,NewPls,dung->PolygonArray,dung->NumPolygons,
                        dung->NumPoints);
            dung->SortingPolygons[i]=dung->PolygonArray[i]=NULL;
            dung->SortingPolygons[i+1]=dung->PolygonArray[i+1]=NULL;
			
            fprintf(stdout,"Read in Polygons\n");
            tx=dung->Points[0]+dung->NumPoints;
            ty=dung->Points[1]+dung->NumPoints;
            tz=dung->Points[2]+dung->NumPoints;
            for(i=0;i<NewPts;i++) {
                fread(tx++,2,1,fp);
                fread(ty++,2,1,fp);
                fread(tz++,2,1,fp);
            }
            fprintf(stdout,"Read in points\n");
            towhat->Next[which]=dung->ObjectArray[dung->NumObjects];
            towhat->Next[which]->Prev=towhat;
            fprintf(stdout,"Attached objects\n");
            dung->NumPoints+=NewPts;
            dung->NumObjects+=NewOs;
            dung->NumPolygons+=NewPls;
        }
    }
}
int
MallocNewShadowPoints (Dungeon *dung,int Number)
{
    dung->shadowpts[0] = (short *)MyMalloc(Number * sizeof (short int),"shadowptsX");
    if (dung->shadowpts[0]) {
        dung->shadowpts[1] = (short *)MyMalloc(Number * sizeof (short int),
                                            "shadowptsY");
        if (dung->shadowpts[1]) {
            dung->shadowpts[2] = (short *)MyMalloc(Number * sizeof (short int),
                                                "shadowptsZ");
            if (dung->shadowpts[2]) {
                return (1);
            }
            MyFree (dung->shadowpts[1], "newshadowptsY");
        }
        MyFree (dung->shadowpts[0], "newshadowptsX");
    }
    return (0);
}

int
MallocShadowPoints (Dungeon *dung,int Number)
{
    dung->newshadowpts[0] = (short *)MyMalloc(Number * sizeof (short int),"newshadowptsX");
    if (dung->newshadowpts[0]) {
        dung->newshadowpts[1] = (short *)MyMalloc(Number * sizeof (short int),
                                            "newshadowptsY");
        if (dung->newshadowpts[1]) {
            dung->newshadowpts[2] = (short *)MyMalloc(Number * sizeof (short int),
                                                "newshadowptsZ");
            if (dung->newshadowpts[2]) {
                return (1);
            }
            MyFree (dung->newshadowpts[1], "newshadowptsY");
        }
        MyFree (dung->newshadowpts[0], "newshadowptsX");
    }
    return (0);
}
 
int 
MallocNewPoints (Dungeon *dung,int Number)
{
    dung->newpts[0] = (short *)MyMalloc(Number * sizeof (short int),"newptsX");
    if (dung->newpts[0]) {
		dung->newpts[1] = (short *)MyMalloc(Number * sizeof (short int), 
											"newptsY");
		if (dung->newpts[1]) {
		    dung->newpts[2] = (short *)MyMalloc(Number * sizeof (short int), 
												"newptsZ");
		    if (dung->newpts[2]) {
				return (1);
		    }
		    MyFree (dung->newpts[1], "newptsY");
		}
		MyFree (dung->newpts[0], "newptsX");
    }
    return (0);
}

int 
MallocPoints (Dungeon *dung,int Number)
{
    dung->Points[0] = (short *)MyMalloc(Number * sizeof (short int),"PointsX");
    if (dung->Points[0]) {
		dung->Points[1] = (short *)MyMalloc(Number * sizeof (short int), 
											"PointsY");
		if (dung->Points[1]) {
		    dung->Points[2] = (short *)MyMalloc(Number * sizeof (short int), 
												"PointsZ");
		    if (dung->Points[2]) {
				return(1);
		    }
		    MyFree(dung->Points[1], "PointsY");
		}
		MyFree(dung->Points[0], "PointsX");
    }
    return(0);
}

int 
LoadPoints (Dungeon *dung,FILE * fp1, FILE * fp2, int Num1, int Num2)
{
    short int *tx, *ty, *tz, i;
    int Number_Of_Points;
    Number_Of_Points = 22 + Num1 + Num2;
	if(MallocPoints(dung,Number_Of_Points)) {
   		tx = dung->Points[0];
   		ty = dung->Points[1];
   		tz = dung->Points[2];
   		for (i = 0; i < 22; i++) {
			*(tx++) = 0;	/* These are the worlds points */
  			*(ty++) = 0;	/*  (and where each object */
		  	*(tz++) = 0;	/* stands in the world) */
		}
		//fprintf(stdout,"Loaded points: %d -> %d\n",22,22+Num1-1);
		for (i = 0; i < Num1; i++) {
			fread (tx++, 2, 1, fp1);
			fread (ty++, 2, 1, fp1);
			fread (tz++, 2, 1, fp1);
		}
		//fprintf(stdout,"Loaded points: %d -> %d\n",22+Num1,
						//						22+Num1+Num2-1);
		for (i = 0; i < Num2; i++) {
			fread (tx++, 2, 1, fp2);
			fread (ty++, 2, 1, fp2);
			fread (tz++, 2, 1, fp2);
		}
		if (MallocNewPoints (dung,Number_Of_Points)) {
			if (MallocShadowPoints(dung,Number_Of_Points)) {
				if (MallocNewShadowPoints(dung,Number_Of_Points)) {
		    		return (1);
				}
			}
		}
		FreePoints(dung);
    }
  	return (0);
}

int 
IsDDC (FILE * fp)
{
    char tmp;
    tmp = fgetc(fp);
    if ('D' == tmp) {
		tmp = fgetc(fp);
		if ('S' == tmp || 'D' == tmp) {
	    	tmp = fgetc(fp);
	    	if ('O' == tmp || 'C' == tmp) {
				if(tmp=='O') {
					fprintf(stdout,"File was a    DSO\n");
				}
				if (DDC_VERSION == (tmp = fgetc (fp))) {
		    		return (1);
				} else {
		    		fprintf (stderr, "Incorrect DDC version: (%d)\n", tmp);
				}
	    	} else {
				fprintf (stderr, "Not a DDC file !! {DS%c(%d)}\n", tmp, fgetc (fp));
			}
		} else {
	    	fprintf (stderr, "Not a DDC file !! {D%c%c(%d)}\n", tmp, fgetc (fp), fgetc (fp));
		}
    } else {
		fprintf (stderr, "Not a DDC file !! {%c%c%c(%d)}\n", tmp, fgetc (fp), fgetc (fp), fgetc (fp));
	}
    fprintf (stderr, "Correct DDC version is \"%d\"\n", DDC_VERSION);
    return(0);
}

void 
LoadColls (int Num,int Colls[MAX_COLLISION_PTS], FILE * fp, int offset)
{
    int i;
    for (i = 0; i < Num; i++) {
		fread (&Colls[i], 4, 1, fp);
		Colls[i] += offset;
	/*	fprintf (stdout, "Collision/Hit Point %d: %d\n", i + 1,
		 Colls[i]);  */
    }
}

FILE *
fopenDDC(char *fname,char *flags) 
{
	FILE *fp;
	fp=fopen(fname,flags);
	if(fp) {
		if(IsDDC(fp)) {
			return(fp);
		} else {
			fprintf(stdout,"Found file, Not a DSO/DDC\n");
		}
		fclose(fp);
	} else {
		fprintf(stdout,"Can't even open it\n");
	}
	return(NULL);
}

int 
MallocHits(Dungeon *dungeon)
{
	dungeon->Colls[0]=MyMalloc(dungeon->NumColls[0]*4,"Colls0");
	if(dungeon->Colls[0]) {
		dungeon->Colls[1]=MyMalloc(dungeon->NumColls[1]*4,"Colls1");
		if(dungeon->Colls[1]) {
			dungeon->Hits[0]=MyMalloc(dungeon->NumHits[0]*4,"Hits0");
			if(dungeon->Hits[0]) {
				dungeon->Hits[1]=MyMalloc(dungeon->NumHits[1]*4,"Hits1");
				if(dungeon->Hits[1]) {
					return(1);
				}
				MyFree(dungeon->Hits[0],"Hits0");
			}
			MyFree(dungeon->Colls[1],"Colls1");
		}
		MyFree(dungeon->Colls[0],"Colls0");
	}
	return(0);
}

struct Bigmove ***
LoadBothMoves(char *fname1,char *fname2)
{
	struct Bigmove ***Moves;
	char tmp[100];
	Moves=MyMalloc(2*sizeof(struct Bigmove **),"PlayersMoves");
	if(Moves) {
		strcpy(tmp,fname1);
		tmp[strlen(tmp)-1]='m';
		tmp[strlen(tmp)-2]='d';
		Moves[0]=LoadMoves(tmp);
		if(Moves[0]) {
			strcpy(tmp,fname2);
			tmp[strlen(tmp)-1]='m';
			tmp[strlen(tmp)-2]='d';
			Moves[1]=LoadMoves(tmp);
			if(Moves[1]) {
				return(Moves);
			} else {
				fprintf(stdout,"Can't Load player 2s moves\n");
			}
			FreeMoves(Moves[0]);
		} else {
			fprintf(stdout,"Can't Load player 1s moves\n");
		}
		MyFree(Moves,"Playermovesarray");
	} else {
		fprintf(stdout,"Can't malloc players move array\n");
	}
	return(NULL);
}

Dungeon *
SetupStuff (char *fname1, char *fname2, Segment *World, 
					int NumText, int PaletteOffset, int *ObjOff, 
					int *npts)
{
	Dungeon *dung;
    FILE *fp1, *fp2;
    int NumO1, NumO2, NumPly1, NumPly2, NumPts1, NumPts2;
	dung=MyMalloc(sizeof(Dungeon),"dungeon mem");
    fp1 = fopenDDC(fname1, "rb");
    if (fp1) {
		fp2 = fopenDDC(fname2, "rb");
		if (fp2) {
			NumO1 = fgetshort(fp1);
			NumO2 = fgetshort(fp2);
			NumPly1 = fgetshort(fp1);
			NumPly2 = fgetshort(fp2);
			NumPts1 = fgetshort(fp1);
			NumPts2 = fgetshort(fp2);
			dung->NumColls[0] = fgetshort(fp1);
			dung->NumColls[1] = fgetshort(fp2);
			dung->NumHits[0] = fgetshort(fp1);
			dung->NumHits[1] = fgetshort(fp2);
			fprintf(stdout," %d:%d  -- %d:%d\n",dung->NumColls[0],dung->NumHits[0],dung->NumColls[1],dung->NumHits[1]);
			*npts = NumPts1 + NumPts2 + 22;
			fprintf(stdout,"%d points\n",*npts);
			dung->NumPoints=*npts;
			dung->ObjectArray = LoadObjects (fp1, fp2, NumO1, NumO2,
											  22, NumPts1 + 22, 0, NumPly1);
			if(dung->ObjectArray) {
				if(GenLoadPolygons(dung,fp1,fp2,NumPly1,NumPly2,22,NumPts1+22)) {
					if (LoadPoints (dung,fp1, fp2, NumPts1, NumPts2)) {
						dung->Player2Pts=NumPts1;
						if(MallocHits(dung)) {
						  LoadColls(dung->NumColls[0],dung->Colls[0], fp1, 22);
						  LoadColls(dung->NumColls[1],dung->Colls[1], fp2, NumPts1+22);
						  LoadColls(dung->NumHits[0],dung->Hits[0], fp1, 22);
						  LoadColls(dung->NumHits[1],dung->Hits[1], fp2, NumPts1+22);
						  dung->feet[0][0]=fgetshort(fp1)+22;
						  dung->feet[0][1]=fgetshort(fp1)+22;
						  dung->feet[1][0]=fgetshort(fp2)+22+NumPts1;
						  dung->feet[1][1]=fgetshort(fp2)+22+NumPts1;
						  fclose (fp1);
						  fclose (fp2);
						  dung->Textures=LoadTextures (NumText);
						  if (dung->Textures) {
								World->Next[0] = dung->ObjectArray[0];
								World->Next[1] = dung->ObjectArray[1];
								World->Next[0]->Next[0] = dung->ObjectArray[2];
								World->Next[1]->Next[0] = 
											dung->ObjectArray[NumO1+2];
								World->Next[0]->CurrZAng = 64;
								World->Next[0]->CurrXAng = 32;
								World->Next[1]->CurrXAng = -32;
								World->CurrXAng = 32 ; // was 28
								World->CurrZAng = 32;
								(dung->Points[2])[11] = -130;
								(dung->Points[2])[17] = -130;

								RemapTextures (dung->Textures, PaletteOffset,
											   NUM_TEXTURES);
								*ObjOff = NumO1;
								dung->Moves=LoadBothMoves(fname1,fname2);
								if(dung->Moves) {
									return (dung);	/* All okay */
								} else {
									fprintf(stdout,"Can't load moves\n");
								}
								FreeTextures(dung->Textures);
						  	} else {
								fprintf(stdout,"Can't load textures\n");
							}
						} else {
							fprintf(stdout,"Can't malloc hits\n");
						}
						FreeAllPoints(dung);
					} else {
						printf("Error: Can't LoadPoints()\n");
					}
				    FreePolygons(dung);
				} else {
				    printf("Error: Can't LoadPolgons()\n");
				}
				FreeObjects(dung->ObjectArray);
	    	} else {
				printf("Error: Can't LoadObjects()\n");
			}
			fclose(fp2);
	   	} else {
			printf ("Error: Can't open \"%s\"\n", fname2);
		}
		fclose (fp1);
    } else {
		printf ("Error: Can't open \"%s\"\n", fname1);
	}
    return (NULL);			/* some sort of ERROR  */
}

void 
FreeColls(int *ptr)
{
	MyFree(ptr,"Colls OR hits");
}

#ifndef SWEET
Dungeon *
SetupStuffed(char *fname,int NumText,int PaletteOffset)
{
	Dungeon *dung;
    FILE *fp;
	dung=MyMalloc(sizeof(Dungeon),"dungeon struct");
	if(dung) {
		fp=fopenDDC(fname,"rb");
		if(fp) {
			dung->NumObjects = fgetshort(fp);
			dung->NumPolygons = fgetshort(fp);
			dung->NumPoints = fgetshort(fp);
			dung->NumColls[0] = fgetshort(fp);
			dung->NumHits[0] = fgetshort(fp);
			if((dung->ObjectArray=LoadObjectsed(fp,dung->NumObjects))) {
				if(GenLoadPolygons(dung,fp,NULL,dung->NumPolygons,0,0,0)) {
					if(LoadPointsed(dung,fp,dung->NumPoints)) {
						if((dung->Colls[0]=MyMalloc(MAX_COLLISION_PTS*4,
													"CollPts"))) {
							LoadColls(dung->NumColls[0],dung->Colls[0], fp, 0);
							if((dung->Hits[0]=MyMalloc(MAX_COLLISION_PTS*4,
														"HitPts"))) {
								LoadColls(dung->NumHits[0],dung->Hits[0], fp, 0);
								dung->feet[0][0]=fgetshort(fp);
								dung->feet[0][1]=fgetshort(fp);
								fclose(fp);
								if((dung->Textures=LoadTextures(NumText))) {
									fprintf(stdout,"Remapping Textures...");
									RemapTextures(dung->Textures, 
												PaletteOffset, NumText);
									fprintf(stdout,"Done !\n");
									return(dung); /* All okay */
								} else fprintf(stdout,"can't load textures\n");
								FreeColls(dung->Hits[0]);
							} else fprintf(stdout,"Can't alloc for Hits\n");
							FreeColls(dung->Colls[0]);
						} else fprintf(stdout,"Can't alloc for Colls\n");
						FreeAllPoints(dung);
					} else printf("Error: Can't LoadPoints()\n");
					FreePolygons(dung);
				} else printf("Error: Can't LoadPolgons()\n");
				FreeObjects(dung->ObjectArray);
			} else printf("Error: Can't LoadObjects()\n");
			fclose(fp);
		} else printf("Error: Can't open \"%s\"\n",fname);
	} else fprintf(stdout,"Error: mem(dungeon)\n");
	return(NULL); /* some sort of ERROR  */
}
#endif
Segment **
LoadObjectsed(FILE *fp,int Number)
{
	struct FILEobject tmp;
	int i,j;
	Segment **object_array;
	object_array = MyMalloc((Number+OBJ_MARG)*sizeof(Segment *),"object ptrs");
	if(object_array) {
		for(i=0;i<Number;i++) {
			object_array[i]=MyMalloc(sizeof(Segment),"object");
			if(!object_array[i])
				break;
		}
		if(i!=Number) {
			for(j=i;j>=0;j--) {
				MyFree(object_array[j],"object");
			}
			MyFree(object_array,"object ptrs");
			return(NULL);
		}
		object_array[Number]=NULL;
		for(i=0;i<Number;i++) {
			fread(&tmp,sizeof(struct FILEobject),1,fp);
			for(j=0;j<5;j++) {
				if(tmp.Next[j]==-1) {
					object_array[i]->Next[j]=NULL;
				} else {
					object_array[i]->Next[j]=object_array[tmp.Next[j]];
				}
			}
			if(tmp.PrevObj==-1) {
				object_array[i]->Prev= NULL;
			} else {
				object_array[i]->Prev=object_array[tmp.PrevObj];
			}
			object_array[i]->Numpoints = tmp.Numpoints;
			object_array[i]->Poffset = tmp.Poff;
			object_array[i]->CurrXAng = 0;
			object_array[i]->CurrYAng = 0;
			object_array[i]->CurrZAng = 0;
			object_array[i]->Moves=NULL;
				object_array[i]->ID=i;
		}
		return(object_array);
	}
	return(NULL);
}

int 
Malloc2DPoints(Dungeon *dung,int Number)
{
    dung->twoDX=MyMalloc(Number*sizeof(short int),"twoDX");
    if(dung->twoDX) {
		dung->twoDY=MyMalloc(Number*sizeof(short int),"twoDY");
		if(dung->twoDY) {
			dung->twoDZ=MyMalloc(Number*sizeof(short int),"twoDZ");
			if(dung->twoDZ) {
				return(1);
			}
			MyFree(dung->twoDY,"twoDY");
		}
		MyFree(dung->twoDX,"twoDX");
    }
    return(0);
}

int 
LoadPointsed(Dungeon *dung,FILE *fp,int Number)
{
    short int *tx,*ty,*tz,i;
    Number+=PTS_MARG;
	if(MallocPoints(dung,Number)) {
    	tx=dung->Points[0];
       	ty=dung->Points[1];
       	tz=dung->Points[2];
       	for(i=0;i<Number-PTS_MARG;i++) {
       		fread(tx++,2,1,fp);
       		fread(ty++,2,1,fp);
       		fread(tz++,2,1,fp);
       	}
       	if(MallocNewPoints(dung,Number)) {
       		if(Malloc2DPoints(dung,Number)) {
       			return(1);
       		}
			FreeNewPoints(dung);
    	}
		FreePoints(dung);
	}
	return(0);
}

