
#include "xstuff.h"
#include "games.h"
#include "graph.h"
#include "pcx.h"
#include "window.h"
#include "mymem.h"
#include "misc.h"

#include <stdio.h>
#include <X11/Xutil.h>

void SetupWindow (Display ** display, Window * win, int argc, char *argv[], 
		  GC * gc, XFontStruct ** font_info, int pixels[20],
		  Colormap * OurColours,int *GotColours)
{
	int i ;
    unsigned long planemasks[20];
	char *display_name = NULL;
	int screen_num;
	struct Picture_Type *PalettePic;
			
	if (!(*display = XOpenDisplay (display_name))) {
		fprintf (stderr, "Cannot connect to X server %s\n",
			 XDisplayName (display_name));
		exit (-1);
	}
	screen_num = DefaultScreen(*display);
	*win = XCreateSimpleWindow(*display, RootWindow(*display, screen_num), 0,
						   0 ,MAX_WINDOW_WIDTH, MAX_WINDOW_HEIGHT, 0,
						   BlackPixel (*display, screen_num), 3);
    XSelectInput(*display, *win, ExposureMask | KeyPressMask |
		 ButtonPressMask | StructureNotifyMask | 0xFFFFFF);

    /*load_font(*display,font_info); */
    *gc = getGC (*display, *win, font_info, screen_num);
    PalettePic = Load_PCX ("palette.pcx", 1);
    if (PalettePic) {
		*OurColours = XDefaultColormap(*display, DefaultScreen (*display));
		if (XAllocColorCells(*display, *OurColours, True, planemasks, 4,
			      				(long*)pixels, 12)) {
			*GotColours=1;
	    	SetPalette3(*display, *OurColours, PalettePic->Palette, 
				 		pixels[0], 192);
		} else {
	    	fprintf (stderr, "Couldn't get colours\n");
			/**OurColours=XCreateColormap(*display,*win,
									DefaultVisual(*display,0)
									,AllocAll);*/
			*OurColours=XCopyColormapAndFree(*display,*OurColours);
			if(*OurColours) {
				XSetWindowColormap(*display,*win,*OurColours);
				XAllocColorCells(*display, *OurColours, True, planemasks, 4,
			      				(long*)pixels, 16);
				pixels[0]=48; // Skip 48 colours.
				*GotColours=2;
	    		SetPalette3(*display, *OurColours, PalettePic->Palette, 
				 		pixels[0], 192);
				// Set the 48 Colours (tie in with Window managers).
				for(i=0;i<48;i++) {


				}
			}
		}
		FreePic(PalettePic);
    } else {
		fprintf(stderr, "Couldn't find palette\n");
    }
    /*XMapWindow(*display,*win); */
}

void RefreshImage(Display *display,GC gc,Window win,XImage *Image)
{
	XPutImage(display,win,gc,Image,0,0,0,0,Image->width,Image->height);
}

void
HideCursor(Display *display,Window win)
{
	Cursor NoCurs;
	Pixmap Source,mask;
	XColor fore_c={0,0,0,0,7,0};
	XColor back_c={0,0,0,0,7,0};
	
	Source=XCreatePixmap(display,win,2,2,1);
	mask=XCreatePixmap(display,win,2,2,1);
	NoCurs=XCreatePixmapCursor(display,Source,mask,&fore_c,&back_c,0,0);
	XDefineCursor(display,win,NoCurs);
	XFreePixmap(display,Source);
	XFreePixmap(display,mask);
}

Window SetupStatsWindow(Display *dis,GC gc,Window Mainwin,XImage **StatsImage,
						int basepix)
{
	Window StatsWin;
	struct Picture_Type *tmp_pic;
	char *mydata;
	StatsWin=XCreateSimpleWindow(dis,Mainwin,0,0,STATS_WINDOW_WIDTH,
								 STATS_WINDOW_HEIGHT,0,4,3);
	XSelectInput(dis,StatsWin,KeyPressMask|ButtonPressMask);
	XMapWindow(dis,StatsWin);
	XSync(dis,False);
	mydata = MyMalloc ((STATS_WINDOW_WIDTH * STATS_WINDOW_HEIGHT)+100000, 
						"ximage-data (statswin)");
    if (mydata) {
        *StatsImage = XCreateImage (dis, XDefaultVisual (dis, 
									DefaultScreen (dis)), 8, ZPixmap, 0, 
									mydata, 1024, STATS_WINDOW_HEIGHT, 
									32, 1024);
        if (*StatsImage) {
			tmp_pic=Load_PCX("textures/health.pcx",0);
			if(tmp_pic) {
				RemapPicture(tmp_pic,basepix,-1);
				CopyArea(tmp_pic->Data,0,0,tmp_pic->Width,(*StatsImage)->data,0,0,1024,352,STATS_WINDOW_HEIGHT);
				CopyArea(tmp_pic->Data,0,0,tmp_pic->Width,(*StatsImage)->data,672,0,1024,352,STATS_WINDOW_HEIGHT);
				FreePic(tmp_pic);
			}
			tmp_pic=Load_PCX("textures/logo.pcx",0);
			if(tmp_pic) {
				RemapPicture(tmp_pic,basepix,-1);
				CopyArea(tmp_pic->Data,0,0,tmp_pic->Width,(*StatsImage)->data,352,0,1024,320,STATS_WINDOW_HEIGHT);
				FreePic(tmp_pic);
			}
			//XAddPixel(*StatsImage,basepix+115);
 			//ClearScreenSWEET(*StatsImage,0,0,1023,STATS_WINDOW_HEIGHT-10);
			//XAddPixel(*StatsImage,basepix+115);
			//LoadImage(*StatsImage,basepix,"textures/logo.pcx",
			//			STATS_WINDOW_WIDTH,STATS_WINDOW_HEIGHT);
		} else {
			fprintf(stdout,"xstuff.c:%d\n",__LINE__);
		}
	} else {
		fprintf(stdout,"xstuff.c:%d\n",__LINE__);
	}
	WaitAWhile(100);  /* This is needed sometimes....
						otherwise, the stats pic mightn't get displayed */
	XPutImage(dis,StatsWin,gc,*StatsImage,0,0,0,0,STATS_WINDOW_WIDTH,
				STATS_WINDOW_HEIGHT);
	//RefreshImage(dis,gc,StatsWin,*StatsImage);
	XSync(dis,False);
	return(StatsWin);
}

void KillStatsWindow(Display *dis,Window StatsWin)
{
	XUnmapWindow(dis,StatsWin);
	XDestroyWindow(dis,StatsWin);
	XSync(dis,False);
}

Window MakeSubWindow (Display *dis, Window MainWin, int num)
{
    static int winpos[4][2] = {
	{0, 60},
	{DEF_WINDOW_WIDTH / 2, 60},
	{0, DEF_WINDOW_HEIGHT / 2 + 30},
	{DEF_WINDOW_WIDTH / 2, DEF_WINDOW_HEIGHT / 2 + 30}
    };
    Window win;
    win = XCreateSimpleWindow(dis, MainWin, winpos[num][0], winpos[num][1],
			      DEF_WINDOW_WIDTH / 2, DEF_WINDOW_HEIGHT / 2 - 30,
			      0, 4, 3);
    XSelectInput (dis, win, ExposureMask | KeyPressMask | ButtonPressMask);
    XMapWindow (dis, win);
    return (win);
}

GC getGC (Display * display, Window win, XFontStruct ** fi, int screen_num)
{
    char *fontname = "12x24";
    GC gc;
    XFontStruct *font_info;
    unsigned long valuemask = 0;
    XGCValues values;
    gc = XCreateGC (display, win, valuemask, &values);
    
    if ((font_info = XLoadQueryFont (display, fontname)) != NULL) {
	XSetFont (display, gc, font_info->fid);
	*fi = font_info;
    } else {
	fprintf (stderr, "Couldn't load font\n");
	*fi = NULL;
    }
    XSetForeground (display, gc, BlackPixel (display, screen_num));
    return (gc);
}
